<?php

namespace Drupal\file_mime_validator\Service;

use Drupal\file\Entity\File;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\HttpFoundation\File\MimeType\FileinfoMimeTypeGuesser as SymfonyFileinfoMimeTypeGuesser;

/**
 * Class File Mime Validator.
 *
 *  @package file_mime_validator
 */
class FileMimeValidator {

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * Logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerChannelFactory;
  /**
   * String translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $translator;

  /**
   * Constructs the file upload secure validation service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service object.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service object.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translator
   *   The string translation service object.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    TranslationInterface $translator) {
    $this->configFactory = $config_factory;
    $this->loggerChannelFactory = $logger_factory;
    $this->translator = $translator;
  }

  /**
   * Check Real Mime function.
   *
   * @param \Drupal\file\Entity\File $file
   *   File.
   *
   * @return array
   *   Return Response Array.
   */
  public function checkRealMime(File $file) {
    // Get mime type from file.
    $mimeByFilename = $file->getMimeType();
    // Get mime type from fileinfo.
    $mimeByFileinfo = (substr($file->getFileUri(), 0, 5) == '/tmp/') ? (new SymfonyFileinfoMimeTypeGuesser())->guess($file->getFileUri()) : 'NOTTMPDIR';

    // If fileinfo agrees with the file's extension, exit.
    if ($mimeByFilename === $mimeByFileinfo) {
      return [];
    }
    // S3 Bucket Case when its finally get uploaded.
    elseif ($mimeByFileinfo != 'NOTTMPDIR') {
      // Logger.
      $this->loggerChannelFactory->get('file_mime_validator')
        ->error("Error while uploading file: MimeTypeGuesser guessed '%mime_by_fileinfo' and fileinfo '%mime_by_filename'", [
          '%mime_by_fileinfo' => $mimeByFileinfo,
          '%mime_by_filename' => $mimeByFilename,
        ]);
      // Error.
      return [
        new TranslatableMarkup('There was a problem with this file. The uploaded file is of type @extension but the real content seems to be @real_extension', [
          '@extension' => $mimeByFilename,
          '@real_extension' => $mimeByFileinfo,
        ], [], $this->translator),
      ];
    }
    else {
      return [];
    }
  }

}
